@@ -1,17 +1,6 @@
-#define PDL_CORE      /* For certain ifdefs */
-#ifndef WIN32
-#define USE_MMAP
-#else
-#undef USE_MMAP
-#endif
-
 #include "pdlcore.h"
 
-#ifdef USE_MMAP
-#include <sys/mman.h>
-#endif
-
-/* Variable storing our the pthread ID for the main PDL thread.
+/* Variable storing the pthread ID for the main PDL thread.
  *  This is used to tell if we are in the main pthread, or in one of
  *  the pthreads spawned for PDL processing
  * This is only used when compiled with pthreads.
@@ -25,9 +14,9 @@ static int done_pdl_main_pthreadID_init = 0;
  *  altogether later
  */
 static char* pdl_pthread_barf_msgs     = NULL;
-static int   pdl_pthread_barf_msgs_len = 0;
+static size_t pdl_pthread_barf_msgs_len = 0;
 static char* pdl_pthread_warn_msgs     = NULL;
-static int   pdl_pthread_warn_msgs_len = 0;
+static size_t pdl_pthread_warn_msgs_len = 0;
 
 #endif
 
@@ -45,8 +34,9 @@ void pdl__magic_add(pdl *it,pdl_magic *mag)
 	mag->next = NULL;
 }
 
-void pdl__magic_rm(pdl *it,pdl_magic *mag)
+pdl_error pdl__magic_rm(pdl *it,pdl_magic *mag)
 {
+        pdl_error PDL_err = {0, NULL, 0};
         pdl_magic **foo = (pdl_magic **)(&(it->magic));
 	int found = 0;
 	while(*foo) {
@@ -59,9 +49,9 @@ void pdl__magic_rm(pdl *it,pdl_magic *mag)
 		}
 	}
 	if( !found ){
-		die("PDL:Magic not found: Internal error\n");
+		return pdl_make_error_simple(PDL_EUSERERROR, "PDL:Magic not found: Internal error\n");
 	}
-	return;
+	return PDL_err;
 }
 
 void pdl__magic_free(pdl *it)
@@ -127,8 +117,6 @@ pdl_magic *pdl__print_magic(pdl *it)
 	  printf("Magic %p\ttype: ",(void*)(*foo));
 		if((*foo)->what & PDL_MAGIC_MARKCHANGED)
 		  printf("PDL_MAGIC_MARKCHANGED");
-		else if ((*foo)->what & PDL_MAGIC_MUTATEDPARENT)
-		  printf("PDL_MAGIC_MUTATEDPARENT");
 		else if ((*foo)->what & PDL_MAGIC_THREADING)
 		  printf("PDL_MAGIC_THREADING");
 		else
@@ -154,16 +142,16 @@ int pdl__ismagic(pdl *it)
 }
 
 static pdl_magic **delayed=NULL;
-static int ndelayed = 0;
+static PDL_Indx ndelayed = 0;
 void pdl_add_delayed_magic(pdl_magic *mag) {
     /* FIXME: Common realloc mistake: 'delayed' nulled but not freed upon failure */
 	delayed = realloc(delayed,sizeof(*delayed)*++ndelayed);
 	delayed[ndelayed-1] = mag;
 }
 void pdl_run_delayed_magic() {
-	int i;
+	PDL_Indx i;
 	pdl_magic **oldd = delayed; /* In case someone makes new delayed stuff */
-	int nold = ndelayed;
+	PDL_Indx nold = ndelayed;
 	delayed = NULL;
 	ndelayed = 0;
 	for(i=0; i<nold; i++) {
@@ -176,13 +164,14 @@ void pdl_run_delayed_magic() {
  *
  * ->bind - magic
  */
-
 void *svmagic_cast(pdl_magic *mag)
 {
 	pdl_magic_perlfunc *magp = (pdl_magic_perlfunc *)mag;
 	dSP;
+	ENTER; SAVETMPS;
 	PUSHMARK(sp);
 	perl_call_sv(magp->sv, G_DISCARD | G_NOARGS);
+	FREETMPS; LEAVE;
 	return NULL;
 }
 
@@ -195,6 +184,7 @@ pdl_magic *pdl_add_svmagic(pdl *it,SV *func)
 {
 	AV *av;
 	pdl_magic_perlfunc *ptr = malloc(sizeof(pdl_magic_perlfunc));
+	if (!ptr) return NULL;
 	ptr->what = PDL_MAGIC_MARKCHANGED | PDL_MAGIC_DELAYED;
 	ptr->vtable = &svmagic_vtable;
 	ptr->sv = newSVsv(func);
@@ -210,41 +200,6 @@ pdl_magic *pdl_add_svmagic(pdl *it,SV *func)
 	return (pdl_magic *)ptr;
 }
 
-
-/****************
- *
- * ->bind - magic
- */
-
-pdl_trans *pdl_find_mutatedtrans(pdl *it)
-{
-	if(!it->magic) return 0;
-	return pdl__call_magic(it,PDL_MAGIC_MUTATEDPARENT);
-}
-
-static void *fammutmagic_cast(pdl_magic *mag)
-{
-	pdl_magic_fammut *magp = (pdl_magic_fammut *)mag;
-	return magp->ftr;
-}
-
-struct pdl_magic_vtable familymutmagic_vtable = {
-	fammutmagic_cast,
-	NULL
-};
-
-pdl_magic *pdl_add_fammutmagic(pdl *it,pdl_trans *ft)
-{
-	pdl_magic_fammut *ptr = malloc(sizeof(pdl_magic_fammut));
-	ptr->what = PDL_MAGIC_MUTATEDPARENT;
-	ptr->vtable = &familymutmagic_vtable;
-	ptr->ftr = ft;
-	ptr->pdl = it;
-	ptr->next = NULL;
-	pdl__magic_add(it,(pdl_magic *)ptr);
-	return (pdl_magic *)ptr;
-}
-
 #ifdef PDL_PTHREAD
 
 /**************
@@ -253,13 +208,12 @@ pdl_magic *pdl_add_fammutmagic(pdl *it,pdl_trans *ft)
  *
  */
 
-#define TVERB 0
-
 typedef struct ptarg {
 	pdl_magic_pthread *mag;
-	void (*func)(pdl_trans *);
+	pdl_error (*func)(pdl_trans *);
 	pdl_trans *t;
 	int no;
+	pdl_error error_return;
 } ptarg;
 
 int pdl_pthreads_enabled(void) {return 1;}
@@ -267,162 +221,148 @@ int pdl_pthreads_enabled(void) {return 1;}
 
 static void *pthread_perform(void *vp) {
 	struct ptarg *p = (ptarg *)vp;
-	/* if(TVERB) printf("STARTING THREAD %d (%d)\n",p->no, pthread_self()); */
-	if(TVERB) printf("STARTING THREAD number %d\n",p->no);
+	PDLDEBUG_f(printf("STARTING THREAD %d (%lu)\n",p->no, (long unsigned)pthread_self()));
 	pthread_setspecific(p->mag->key,(void *)&(p->no));
-	(p->func)(p->t);
-	/* if(TVERB) printf("ENDING THREAD %d (%d)\n",p->no, pthread_self());   */
-	if(TVERB) printf("ENDING THREAD number %d\n",p->no);
+	p->error_return = (p->func)(p->t);
+	PDLDEBUG_f(printf("ENDING THREAD %d (%lu)\n",p->no, (long unsigned)pthread_self()));
 	return NULL;
 }
 
-int pdl_magic_thread_nthreads(pdl *it,int *nthdim) {
+int pdl_magic_thread_nthreads(pdl *it, PDL_Indx *nthdim) {
 	pdl_magic_pthread *ptr = (pdl_magic_pthread *)pdl__find_magic(it, PDL_MAGIC_THREADING);
 	if(!ptr) return 0;
-	*nthdim = ptr->nthdim;
+	if (nthdim) *nthdim = ptr->nthdim;
 	return ptr->nthreads;
 }
 
-int pdl_magic_get_thread(pdl *it) { /* XXX -> only one thread can handle pdl at once */
-	pdl_magic_pthread *ptr;
-	int *p;
-	ptr = (pdl_magic_pthread *)pdl__find_magic(it, PDL_MAGIC_THREADING);
-	if(!ptr) {die("Invalid pdl_magic_get_thread!");}
-	p = (int*)pthread_getspecific(ptr->key);
-	if(!p) {
-		die("Invalid pdl_magic_get_thread specific!!!!");
-	}
+int pdl_magic_get_thread(pdl *it) {
+	pdl_magic_pthread *ptr = (pdl_magic_pthread *)pdl__find_magic(it, PDL_MAGIC_THREADING);
+	if(!ptr) return -1;
+	int *p = (int*)pthread_getspecific(ptr->key);
+	if(!p) return -1;
 	return *p;
 }
 
-void pdl_magic_thread_cast(pdl *it,void (*func)(pdl_trans *),pdl_trans *t, pdl_thread *thread) {
-	pdl_magic_pthread *ptr; pthread_t *tp; ptarg *tparg;
-	int i;
+pdl_error pdl_magic_thread_cast(pdl *it,pdl_error (*func)(pdl_trans *),pdl_trans *t, pdl_broadcast *broadcast) {
+	pdl_error PDL_err = {0, NULL, 0};
+	PDL_Indx i;
 	int clearMagic = 0; /* Flag = 1 if we are temporarily creating pthreading magic in the
 						   supplied pdl.  */
-	SV * barf_msg;	  /* Deferred barf message. Using a perl SV here so it's memory can be freeed by perl
-						 after it is sent to croak */
-	SV * warn_msg;	  /* Similar deferred warn message. */
-
-	ptr = (pdl_magic_pthread *)pdl__find_magic(it, PDL_MAGIC_THREADING);
+	pdl_magic_pthread *ptr = (pdl_magic_pthread *)pdl__find_magic(it, PDL_MAGIC_THREADING);
 	if(!ptr) {
 		/* Magic doesn't exist, create it
 			Probably was deleted before the transformation performed, due to
 			pdl lazy evaluation.
 		*/
 
-		pdl_add_threading_magic(it, thread->mag_nth, thread->mag_nthr);
+		PDL_RETERROR(PDL_err, pdl_add_threading_magic(it, broadcast->mag_nth, broadcast->mag_nthr));
 		clearMagic = 1; /* Set flag to delete magic later */
 
 		/* Try to get magic again */
 		ptr = (pdl_magic_pthread *)pdl__find_magic(it, PDL_MAGIC_THREADING);
 
-		if(!ptr) {die("Invalid pdl_magic_thread_cast!");}
+		if(!ptr) {return pdl_make_error_simple(PDL_EFATAL, "Invalid pdl_magic_thread_cast!");}
 
 	}
 
-	tp = malloc(sizeof(pthread_t) * thread->mag_nthr);
-	tparg = malloc(sizeof(*tparg) * thread->mag_nthr);
+	pthread_t tp[broadcast->mag_nthr];
+	ptarg tparg[broadcast->mag_nthr];
 	pthread_key_create(&(ptr->key),NULL);
-
-	if(TVERB) printf("CREATING THREADS, ME: TBD, key: %ld\n", (unsigned long)(ptr->key));
-
 	/* Get the pthread ID of this main thread we are in.
 	 *	Any barf, warn, etc calls in the spawned pthreads can use this
 	 *	to tell if its a spawned pthread
 	 */
-	pdl_main_pthreadID = pthread_self();   /* should do inside pthread_once() */
-    done_pdl_main_pthreadID_init = 1;
+	pdl_main_pthreadID = pthread_self();
+	done_pdl_main_pthreadID_init = 1;
 
-    for(i=0; i<thread->mag_nthr; i++) {
-        tparg[i].mag = ptr;
-        tparg[i].func = func;
-        tparg[i].t = t;
-        tparg[i].no = i;
-        if (pthread_create(tp+i, NULL, pthread_perform, tparg+i)) {
-            die("Unable to create pthreads!");
-        }
-    }
-
-    if(TVERB) printf("JOINING THREADS, ME: TBD, key: %ld\n", (unsigned long)(ptr->key));
-
-	for(i=0; i<thread->mag_nthr; i++) {
-		pthread_join(tp[i], NULL);
+	PDLDEBUG_f(printf("CREATING THREADS, ME: TBD, key: %ld\n", (unsigned long)(ptr->key)));
+	for(i=0; i<broadcast->mag_nthr; i++) {
+	    tparg[i].mag = ptr;
+	    tparg[i].func = func;
+	    tparg[i].t = t;
+	    tparg[i].no = i;
+	    tparg[i].error_return = PDL_err;
+	    if (pthread_create(tp+i, NULL, pthread_perform, tparg+i)) {
+		return pdl_make_error_simple(PDL_EFATAL, "Unable to create pthreads!");
+	    }
 	}
 
-	if(TVERB) printf("FINISHED THREADS, ME: TBD, key: %ld\n", (unsigned long)(ptr->key));
+	PDLDEBUG_f(printf("JOINING THREADS, ME: TBD, key: %ld\n", (unsigned long)(ptr->key)));
+	for(i=0; i<broadcast->mag_nthr; i++) {
+		pthread_join(tp[i], NULL);
+	}
+	PDLDEBUG_f(printf("FINISHED THREADS, ME: TBD, key: %ld\n", (unsigned long)(ptr->key)));
 
 	pthread_key_delete((ptr->key));
+	done_pdl_main_pthreadID_init = 0;
 
 	/* Remove pthread magic if we created in this function */
 	if( clearMagic ){
-		pdl_add_threading_magic(it, -1, -1);
+		PDL_RETERROR(PDL_err, pdl_add_threading_magic(it, -1, -1));
 	}
 
-	/* Clean up memory allocated */
-	free(tp);
-	free(tparg);
-
-	// handle any errors that may have occured in the worker threads I reset the
-	// length before actually barfing/warning because barf() may not come back.
-	// In that case, I'll have len==0, but an unfreed pointer. This memory will
-	// be reclaimed the next time we barf/warn something (since I'm using
-	// realloc). If we never barf/warn again, we'll hold onto this memory until
-	// the interpreter exits. This is a one-time penalty, though so it's fine
-#define handle_deferred_errors(type)							\
+#define handle_deferred_errors(type, action)							\
 	do{															\
 		if(pdl_pthread_##type##_msgs_len != 0)					\
 		{														\
 			pdl_pthread_##type##_msgs_len = 0;					\
-			pdl_##type ("%s", pdl_pthread_##type##_msgs);		\
+			action;	\
 			free(pdl_pthread_##type##_msgs);					\
 			pdl_pthread_##type##_msgs	  = NULL;				\
 		}														\
 	} while(0)
 
-	handle_deferred_errors(warn);
-	handle_deferred_errors(barf);
+	handle_deferred_errors(warn, pdl_pdl_warn("%s", pdl_pthread_warn_msgs));
+	handle_deferred_errors(barf, PDL_err = pdl_error_accumulate(PDL_err, pdl_make_error(PDL_EFATAL, "%s", pdl_pthread_barf_msgs)));
+	for(i=0; i<broadcast->mag_nthr; i++) {
+	    PDL_err = pdl_error_accumulate(PDL_err, tparg[i].error_return);
+	}
+	return PDL_err;
 }
 
 /* Function to remove threading magic (added by pdl_add_threading_magic) */
-void pdl_rm_threading_magic(pdl *it)
+pdl_error pdl_rm_threading_magic(pdl *it)
 {
+	pdl_error PDL_err = {0, NULL, 0};
 	pdl_magic_pthread *ptr = (pdl_magic_pthread *)pdl__find_magic(it, PDL_MAGIC_THREADING);
-
 	/* Don't do anything if threading magic not found */
-	if( !ptr) return;
-
+	if( !ptr) return PDL_err;
 	/* Remove magic */
-	pdl__magic_rm(it, (pdl_magic *) ptr);
-
+	PDL_RETERROR(PDL_err, pdl__magic_rm(it, (pdl_magic *) ptr));
 	/* Free magic */
 	free( ptr );
+	return PDL_err;
 }
 
 /* Function to add threading magic (i.e. identify which PDL dimension should
    be pthreaded and how many pthreads to create
    Note: If nthdim and nthreads = -1 then any pthreading magic is removed */
-void pdl_add_threading_magic(pdl *it,int nthdim,int nthreads)
+pdl_error pdl_add_threading_magic(pdl *it,PDL_Indx nthdim,PDL_Indx nthreads)
 {
-      pdl_magic_pthread *ptr;
-
+	pdl_error PDL_err = {0, NULL, 0};
+	pdl_magic_pthread *ptr;
 	/* Remove threading magic if called with parms -1, -1 */
 	if( (nthdim == -1) && ( nthreads == -1 ) ){
-		 pdl_rm_threading_magic(it);
-		 return;
+		 PDL_RETERROR(PDL_err, pdl_rm_threading_magic(it));
+		 return PDL_err;
 	}
-
 	ptr = malloc(sizeof(pdl_magic_pthread));
+	if (!ptr) return pdl_make_error_simple(PDL_EFATAL, "Out of memory");
 	ptr->what = PDL_MAGIC_THREADING;
 	ptr->vtable = NULL;
 	ptr->next = NULL;
 	ptr->nthdim = nthdim;
 	ptr->nthreads = nthreads;
 	pdl__magic_add(it,(pdl_magic *)ptr);
+	return PDL_err;
+}
+
+char pdl_pthread_main_thread() {
+  return !done_pdl_main_pthreadID_init || pthread_equal( pdl_main_pthreadID, pthread_self() );
 }
 
 // Barf/warn function for deferred barf message handling during pthreading We
-// can't barf/warn during ptheading, because perl-level code isn't
+// can't barf/warn during pthreading, because perl-level code isn't
 // threadsafe. This routine does nothing if we're in the main thread (allowing
 // the caller to barf normally, since there are not threading issues then). If
 // we're in a worker thread, this routine stores the message for main-thread
@@ -430,51 +370,25 @@ void pdl_add_threading_magic(pdl *it,int nthdim,int nthreads)
 int pdl_pthread_barf_or_warn(const char* pat, int iswarn, va_list *args)
 {
 	char** msgs;
-	int*   len;
+	size_t* len;
 
 	/* Don't do anything if we are in the main pthread */
-	if( !done_pdl_main_pthreadID_init || pthread_equal( pdl_main_pthreadID, pthread_self() ) )
-		return 0;
+	if (pdl_pthread_main_thread()) return 0;
 
 	if(iswarn)
 	{
 		msgs = &pdl_pthread_warn_msgs;
-		len	 = &pdl_pthread_warn_msgs_len;
+		len = &pdl_pthread_warn_msgs_len;
 	}
 	else
 	{
 		msgs = &pdl_pthread_barf_msgs;
-		len	 = &pdl_pthread_barf_msgs_len;
+		len = &pdl_pthread_barf_msgs_len;
 	}
 
+	size_t extralen = vsnprintf(NULL, 0, pat, *args);
 	// add the new complaint to the list
-	{
-		static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
-		pthread_mutex_lock( &mutex );
-		{
-			/* In the chunk I'm adding I need to store the actual data and trailing newline. */
-			int extralen = vsnprintf(NULL, 0, pat, *args) + 1;
-
-			/* 1 more for the trailing '\0'. (For windows, we first #undef realloc
-			   so that the system realloc function is used instead of the PerlMem_realloc
-			   macro. This currently works fine, though could conceivably require some
-			   tweaking in the future if it's found to cause any problem.) */
-#ifdef WIN32
-#undef realloc
-#endif
-            /* FIXME: Common realloc mistake: 'msgs' nulled but not freed upon failure */
-			*msgs = realloc(*msgs, *len + extralen + 1);
-			vsnprintf( *msgs + *len, extralen + 1, pat, *args);
-
-			/* update the length-so-far. This does NOT include the trailing '\0' */
-			*len += extralen;
-
-			/* add the newline to the end */
-			(*msgs)[*len-1] = '\n';
-			(*msgs)[*len  ] = '\0';
-		}
-		pthread_mutex_unlock( &mutex );
-	}
+	pdl_pthread_realloc_vsnprintf(msgs, len, extralen, pat, args, 1);
 
 	if(iswarn)
 	{
@@ -487,15 +401,98 @@ int pdl_pthread_barf_or_warn(const char* pat, int iswarn, va_list *args)
 	return 0;
 }
 
+void pdl_pthread_realloc_vsnprintf(char **p, size_t *len, size_t extralen, const char *pat, va_list *args, char add_newline) {
+  static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
+  pthread_mutex_lock( &mutex );
+  /* (For windows, we first #undef realloc
+     so that the system realloc function is used instead of the PerlMem_realloc
+     macro. This currently works fine, though could conceivably require some
+     tweaking in the future if it's found to cause any problem.) */
+#ifdef WIN32
+#undef realloc
+#endif
+  if (add_newline) extralen += 1;
+  extralen += 1; /* +1 for '\0' at end */
+  *p = realloc(*p, *len + extralen);
+  vsnprintf(*p + *len, extralen, pat, *args);
+  *len += extralen; /* update the length-so-far, includes '\0' */
+  if (add_newline) (*p)[*len-2] = '\n';
+  (*p)[*len-1] = '\0';
+  pthread_mutex_unlock( &mutex );
+}
+
+void pdl_pthread_free(void *p) {
+#ifdef WIN32 /* same reasons as above */
+#undef free
+#endif
+  static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
+  pthread_mutex_lock( &mutex );
+  free(p);
+  pthread_mutex_unlock( &mutex );
+}
+
+/* copied from git@github.com:git/git.git 2.34-ish thread-util.c */
+/* changed GIT_WINDOWS_NATIVE to WIN32 */
+#if defined(hpux) || defined(__hpux) || defined(_hpux)
+#  include <sys/pstat.h>
+#endif
+/*
+ * By doing this in two steps we can at least get
+ * the function to be somewhat coherent, even
+ * with this disgusting nest of #ifdefs.
+ */
+#ifndef _SC_NPROCESSORS_ONLN
+#  ifdef _SC_NPROC_ONLN
+#    define _SC_NPROCESSORS_ONLN _SC_NPROC_ONLN
+#  elif defined _SC_CRAY_NCPU
+#    define _SC_NPROCESSORS_ONLN _SC_CRAY_NCPU
+#  endif
+#endif
+int pdl_online_cpus(void)
+{
+#ifdef WIN32
+	SYSTEM_INFO info;
+	GetSystemInfo(&info);
+	if ((int)info.dwNumberOfProcessors > 0)
+		return (int)info.dwNumberOfProcessors;
+#elif defined(hpux) || defined(__hpux) || defined(_hpux)
+	struct pst_dynamic psd;
+	if (!pstat_getdynamic(&psd, sizeof(psd), (size_t)1, 0))
+		return (int)psd.psd_proc_cnt;
+#elif defined(HAVE_BSD_SYSCTL) && defined(HW_NCPU)
+	int mib[2];
+	size_t len;
+	int cpucount;
+	mib[0] = CTL_HW;
+#  ifdef HW_AVAILCPU
+	mib[1] = HW_AVAILCPU;
+	len = sizeof(cpucount);
+	if (!sysctl(mib, 2, &cpucount, &len, NULL, 0))
+		return cpucount;
+#  endif /* HW_AVAILCPU */
+	mib[1] = HW_NCPU;
+	len = sizeof(cpucount);
+	if (!sysctl(mib, 2, &cpucount, &len, NULL, 0))
+		return cpucount;
+#endif /* defined(HAVE_BSD_SYSCTL) && defined(HW_NCPU) */
+#ifdef _SC_NPROCESSORS_ONLN
+	long ncpus;
+	if ((ncpus = (long)sysconf(_SC_NPROCESSORS_ONLN)) > 0)
+		return (int)ncpus;
+#endif
+	return 1;
+}
 
 #else
 /* Dummy versions */
-void pdl_add_threading_magic(pdl *it,int nthdim,int nthreads) {}
+pdl_error pdl_add_threading_magic(pdl *it,PDL_Indx nthdim,PDL_Indx nthreads) {pdl_error PDL_err = {0,NULL,0}; return PDL_err;}
+char pdl_pthread_main_thread() { return 1; }
 int pdl_magic_get_thread(pdl *it) {return 0;}
-void pdl_magic_thread_cast(pdl *it,void (*func)(pdl_trans *),pdl_trans *t, pdl_thread *thread) {}
-int pdl_magic_thread_nthreads(pdl *it,int *nthdim) {return 0;}
+pdl_error pdl_magic_thread_cast(pdl *it,pdl_error (*func)(pdl_trans *),pdl_trans *t, pdl_broadcast *broadcast) {pdl_error PDL_err = {0,NULL,0}; return PDL_err;}
+int pdl_magic_thread_nthreads(pdl *it,PDL_Indx *nthdim) {return 0;}
 int pdl_pthreads_enabled() {return 0;}
 int pdl_pthread_barf_or_warn(const char* pat, int iswarn, va_list *args){ return 0;}
+int pdl_online_cpus() {return 1;}
 #endif
 
 /***************************
@@ -504,19 +501,6 @@ int pdl_pthread_barf_or_warn(const char* pat, int iswarn, va_list *args){ return
  *
  */
 
-
-void pdl_delete_mmapped_data(pdl *p, Size_t param)
-{
-	if(!p) {return;}
-	if(!p->data) {return;}
-#ifdef USE_MMAP
-	munmap(p->data, param);
-#else
-      /*  croak("internal error: trying to delete mmaped data on unsupported platform"); */
-#endif
-	p->data = 0;
-}
-
 static void *delete_mmapped_cast(pdl_magic *mag)
 {
 	pdl_magic_deletedata *magp = (pdl_magic_deletedata *)mag;
@@ -529,14 +513,16 @@ struct pdl_magic_vtable deletedatamagic_vtable = {
 	NULL
 };
 
-void pdl_add_deletedata_magic(pdl *it, void (*func)(pdl *, Size_t param), Size_t param)
+pdl_error pdl_add_deletedata_magic(pdl *it, void (*func)(pdl *, Size_t param), Size_t param)
 {
+	pdl_error PDL_err = {0, NULL, 0};
 	pdl_magic_deletedata *ptr = malloc(sizeof(pdl_magic_deletedata));
+	if (!ptr) return pdl_make_error_simple(PDL_EFATAL, "Out of memory");
 	ptr->what = PDL_MAGIC_DELETEDATA;
 	ptr->vtable = &deletedatamagic_vtable;
 	ptr->pdl = it;
 	ptr->func = func;
 	ptr->param = param;
 	pdl__magic_add(it, (pdl_magic *)ptr);
+	return PDL_err;
 }
-
