@@ -283,7 +283,7 @@ sub PDL::Graphics::TriD::Image::vdraw {
   my ($this,$vert) = @_;
   my $p = $this->flatten(0); # no binary alignment
   if(!defined $vert) {$vert = $this->{Points}}
-  my $url = new PDL::Graphics::TriD::VRML::URL('image/JPG');
+  my $url = PDL::Graphics::TriD::VRML::URL->new('image/JPG');
   $url->save($p);
   vrn('Shape',
       'appearance' => vrn('Appearance',
@@ -365,7 +365,7 @@ sub PDL::Graphics::TriD::EuclidAxes::tovrml_axis {
 sub PDL::Graphics::TriD::SimpleController::tovrml {
   # World origin is disregarded XXXXXXX
   my $this = shift;
-  my $inv = new PDL::Graphics::TriD::Quaternion(@{$this->{WRotation}});
+  my $inv = PDL::Graphics::TriD::Quaternion->new(@{$this->{WRotation}});
   $inv->invert_rotation_this;
   my $pos = $inv->rotate([0,0,1]);
 #  print "SC: POS0:",(join ',',@$pos),"\n";
@@ -411,7 +411,7 @@ sub checkerr {
 }
 
 sub activate {
-  my $client = new Win32::DDE::Client ('Netscape','WWW_Activate');
+  my $client = Win32::DDE::Client->new('Netscape','WWW_Activate');
   checkerr($client);
   $client->Request('0xFFFFFFFF,0x0');
   barf "can't disconnect" unless $client->Disconnect;
@@ -419,7 +419,7 @@ sub activate {
 
 sub geturl {
   my ($url) = @_;
-  my $client = new Win32::DDE::Client ('Netscape','WWW_OpenURL');
+  my $client = Win32::DDE::Client->new('Netscape','WWW_OpenURL');
   checkerr($client);
   my $status = $client->Request("\"$url\",,0xFFFFFFFF,0x1");
   barf "can't disconnect" unless $client->Disconnect;
@@ -609,10 +609,10 @@ sub gdriver {
 
   require PDL if not defined $PDL::VERSION;
   $this->{Width} = 300; $this->{Height} = 300;
-  $this->{VRMLTop} = new PDL::Graphics::VRML("\"PDL::Graphics::TriD::VRML Scene\"",
+  $this->{VRMLTop} = PDL::Graphics::VRML->new("\"PDL::Graphics::TriD::VRML Scene\"",
 				  ["\"generated by the PDL::Graphics::TriD module\"",
 				   "\"version $PDL::VERSION\""]);
-  my $fontstyle = new PDL::Graphics::VRMLNode('FontStyle',
+  my $fontstyle = PDL::Graphics::VRMLNode->new('FontStyle',
 				    'size' => 0.04,
 				    'family' => "\"SANS\"",
 				    'justify' => "\"MIDDLE\"");
@@ -649,7 +649,7 @@ sub new_viewport {
 	barf "only allowing [0,1,0,1] viewports with VRML backend"
 	  if abs(PDL->pdl($x0,$y0,$x1-1,$y1-1))->max > 0.01;
 
-	my $vp = new PDL::Graphics::TriD::ViewPort($x0,$y0,$x1,$y1);
+	my $vp = PDL::Graphics::TriD::ViewPort->new($x0,$y0,$x1,$y1);
 	push @{$this->{_ViewPorts}},$vp;
 	return $vp;
 }
